﻿using System;
using System.Collections.Generic;
using System.Web.Http.SelfHost;
using System.Windows.Forms;
using DicomObjects;
using DicomObjects.DicomWeb;
using System.Linq;
using DicomObjects.Enums;
using System.IO;

namespace DicomWebDemo
{
    // PLEASE MAKE SURE YOU UNBLOCK ALL DLLS FROM THE DOWNLOAD ZIP FILE
    // PLEASE COPY ALL avXYZ..DLLs from packages/DicomObjects folder to your debug folder
    // PLEASE COPY ALL swXYZ..DLLs from packages/DicomObjects folder to your debug folder

    public partial class Form1 : Form
    {
        DicomServer InternalServer;

#if DEBUG
        public static string BaseUri = "http://localhost:9977";
#else
        public static string BaseUri = "http://dicomserver.co.uk:81";
#endif

        public Form1()
        {
            InitializeComponent();
            mainform = this;
            DicomGlobal.IndentJson = true;
            InternalServer = new DicomServer();
            InternalServer.Listen(888);
            InternalServer.DefaultStatus = 0;
            InternalServer.InstanceReceived += (x, y) => dicomViewer1.Images.Add(new DicomImage(y.Instance) { CineMode = CineMode.Repeat });
        }

        static Form1 mainform;

        HttpSelfHostServer listen;

        DicomWebServer server = new DicomWebServer();

        private void Form1_Load(object sender, EventArgs e)
        {
            HttpSelfHostConfiguration config = new HttpSelfHostConfiguration(BaseUri);

            server.Configure(config, BaseUri, "stow", "qido", "wado", "bulk");

            server.StowReceived += Server_StowReceived;
            server.QidoReceived += Server_QidoReceived;
            server.WadoReceived += Server_WadoReceived;

            listen = new HttpSelfHostServer(config);
            listen.OpenAsync().Wait();
        }

        private void Server_WadoReceived(WadoEventArgs args)
        {
            bool useFrameLevel = true; // this depends on the server

            DicomDataSetCollection results;
            DicomQuery q = args.MakeQuery(useFrameLevel);

            q.Node = Properties.Settings.Default.DICOMServerIPAddress;
            q.Port = Properties.Settings.Default.DICOMServerPortNumber;
            q.CalledAE = Properties.Settings.Default.DICOMServerAET;
            q.CallingAE = "WADO_Client";

            results = q.Get();
            args.ReturnData(results, useFrameLevel);
        }

        private void Server_QidoReceived(QidoEventArgs args)
        {
            DicomQuery q = new DicomQuery();

            q.Node = Properties.Settings.Default.DICOMServerIPAddress;
            q.Port = Properties.Settings.Default.DICOMServerPortNumber;
            q.CallingAE = "QIDO_Client";
            q.CalledAE = Properties.Settings.Default.DICOMServerAET;
            
            q.Root = QueryRoot.Study;
            q.Level = args.Level;

            // do DICOM C-FIND
            args.Results = q.Find(args.QueryDataSet());

            if (args.FuzzyMatching)
                args.Warnings.Add("The fuzzymatching parameter is not supported. Only literal matching has been performed.");

        }

        private void Server_StowReceived(StowEventArgs args)
        {
            // this is just a sample - sending on elsewhere
            var results = args.DataSets;

            foreach (DicomDataSet d in results)
            {
                try
                {
                    int status;
                    using (DicomAssociation cn = new DicomAssociation())
                    {
                        cn.RequestedContexts.Add(d.SOPClass);

                        cn.Open("localhost", 888, "STOW_Client", "me");

                        cn.SendInstances(d);
                        status = cn.LastStatus;
                        
                        cn.Close();
                    }

                    if (status == 0)
                        args.SetSuccess(d);
                    else // treat non-zero status as failure
                        args.SetFailed(d, status);
                }
                catch (Exception)
                {
                    args.SetFailed(d, 0xA900);
                }
            }
        }      

        // new STOW
        private void button1_Click(object sender, EventArgs e)
        {
            var dss = new DicomDataSetCollection();
            dss.Read(Properties.Resources.im1);
            var dwc = new StowWebClient($"{BaseUri}/stow/");
            var r = dwc.Store(dss, DicomDataEncoding.Json, TransferSyntax.Text);
        }

        // new Qido
        private void button3_Click(object sender, EventArgs e)
        {           
            var dwc = new QidoWebClient($"{BaseUri}/qido");
            var q = new DicomQuery();

            q.Level = QueryLevel.STUDY;
            q.Root = QueryRoot.Study;
            
            DicomDataSet ds = q.QueryDataSet();
            ds.Add(Keyword.PatientID, "PAT011");

            var result = dwc.Query(ds);
            MessageBox.Show($"Hits = {result.Count}");
        }

        // new WADO - native
        private void button4_Click(object sender, EventArgs e)
        {            
            var dwc = new WadoWebClient($"{BaseUri}/wado");
            var result = dwc.RetrieveNative("1.3.46.670589.7.1.10.80111056120.19960321.105144.1");
            AddImagesWithCine(result);
        }

        // new WADO - metadata only (with auto fetching of missing bulk items)
        private void button5_Click(object sender, EventArgs e)
        {
            var dwc = new WadoWebClient($"{BaseUri}/wado");            
            dwc.TransferSyntaxes = new List<string> { "1.2.840.10008.1.2.1" };
            var result = dwc.RetrieveMetaData("1.3.46.670589.7.1.10.80111056120.19960321.105144.1", "1.3.46.670589.7.1.8.80111056120.19960321.105144.1", "1.3.46.670589.7.1.3.80111056120.19960321.105144.1");
            AddImagesWithCine(result);
        }

        // new Wado - get data first then merge (as one go - more efficient that separate steps)
        private void button6_Click(object sender, EventArgs e)
        {
            var dwc = new WadoWebClient($"{BaseUri}/wado");
            dwc.TransferSyntaxes[0] = "1.2.840.10008.1.2.4.50";
            var result = dwc.RetrieveMetaData("1.3.46.670589.7.1.10.80111056120.19960321.105144.1", "1.3.46.670589.7.1.8.80111056120.19960321.105144.1", "1.3.46.670589.7.1.3.80111056120.19960321.105144.1");

            dwc.AppendBulkData(result);
            AddImagesWithCine(result);
        }

        // new WADO - rendered
        private void button7_Click(object sender, EventArgs e)
        {
            var dwc = new WadoWebClient($"{BaseUri}/wado");            
            var result = dwc.RetrieveRendered("1.3.46.670589.7.1.10.80111056120.19960321.105144.1", "1.3.46.670589.7.1.8.80111056120.19960321.105144.1", "1.3.46.670589.7.1.3.80111056120.19960321.105144.1");
            
            // this is single only for now#
            var doc = result.First();

            string filename = "";
            var z = doc.Key.ContentType.MediaType;

            if (z.StartsWith("image"))
                filename = z.Substring(6);

            else if (z.StartsWith("application"))
                filename = z.Substring(12);

            else if (z.StartsWith("video"))
                filename = z.Substring(6);

            filename = "c:\\t1." + filename;
            File.WriteAllBytes(filename, doc.Value);

            webBrowser1.Navigate(filename);
        }

        // test stuff
        public static void AddImagesWithCine(IEnumerable<DicomDataSet> results)
        {
            foreach (DicomDataSet d in results)
            {
                d.GetAllBulkData();
                var img = new DicomImage(d);
                if (img.FrameCount > 1)
                    img.CineMode = CineMode.Repeat;                
                mainform.dicomViewer1.Images.Add(img);
            }
        }

        private void Clear_Click(object sender, EventArgs e)
        {
            dicomViewer1.Images.Clear();
        }

    }
}

